from .. operators.dont_register.checks import there_are_cache_nodes
import bpy
from bpy.app.handlers import persistent
from .. operators.dont_register import comp_operations, scene_helpers,checks
from ..variables import addon_variables

@persistent
def my_load(*args):
    addon_variables.reset_all_vars()
    print('reset vars in load event')

def can_update_images(scene_name):
    if bpy.app.is_job_running('RENDER'):
        print('cancelling out of frame change handler because rendering')
        return False
    
    if addon_variables.skip_all_frame_change_handlers:
        print('skip all frame change handlers is set to True')
        return False

    if 'cache_render' in bpy.data.scenes[scene_name]:
        print('cache render scene property is true, skipping frame change handler')
        return False
    
    return True


def select_the_viewer_node(scene_name):
    v = [n.name for n in bpy.data.scenes[scene_name].node_tree.nodes if n.type =='VIEWER']
    bpy.data.scenes[scene_name].node_tree.nodes[v[0]].select = True


@persistent
def update_on_frame_change(scene):
    #print(f'frame change handler.  frame = {scene.frame_current}.')
    if not can_update_images(scene.name):
        print('cant update frame')
        return
    
    addon_variables.skip_all_frame_change_handlers = True

    if bpy.context.screen.is_animation_playing and not bpy.context.screen.is_scrubbing:
        select_the_viewer_node(scene.name)
        try:            
            if addon_variables.nodes_muted_at_beginning_of_playback is None:
                scene_helpers.mute_what_can_be_muted_before_playback(scene.name)
            scene_helpers.update_rendered_image_paths(scene.name)
        except Exception as e:
            print(e)
            bpy.ops.screen.animation_cancel()

    else:
        #manual frame change or playback just stopped
        #mute nodes to avoid 3d viewport rendering
        if addon_variables.nodes_muted_at_beginning_of_playback is None:
            #it's a manual frame change
            scene_helpers.mute_what_can_be_muted_before_playback(scene.name)
            scene_helpers.update_rendered_image_paths(scene.name)
        
        unmute_muted(scene.name)
        

    addon_variables.skip_all_frame_change_handlers = None

def unmute_muted(scene_name):
    scene_helpers.un_mute_playback_muted_cache_nodes(scene_name,True)

@persistent
def playback_stopped(scene):
    unmute_muted(scene.name)

@persistent
def render_pre(scene):
    addon_variables.actual_frame = scene.frame_current

def register_handler():
   

    if update_on_frame_change not in bpy.app.handlers.frame_change_post:
        bpy.app.handlers.frame_change_post.append(update_on_frame_change)

    if my_load not in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.append(my_load)

    if render_pre not in bpy.app.handlers.render_pre:
        bpy.app.handlers.render_pre.append(render_pre)
    
    if hasattr(bpy.app.handlers,'animation_playback_post'):
        if playback_stopped not in bpy.app.handlers.animation_playback_post:
            bpy.app.handlers.animation_playback_post.append(playback_stopped)

def unregister_handler():
    
    if update_on_frame_change in bpy.app.handlers.frame_change_post:
        bpy.app.handlers.frame_change_post.remove(update_on_frame_change)
    if my_load in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.remove(my_load)

    if render_pre in bpy.app.handlers.render_pre:
        bpy.app.handlers.render_pre.remove(render_pre)

    if hasattr(bpy.app.handlers,'animation_playback_post'):
        if playback_stopped in bpy.app.handlers.animation_playback_post:
            bpy.app.handlers.animation_playback_post.remove(playback_stopped)
