from . import node_tools
from . import file_tools
import bpy

def get_original_node_name_from_image_block_name(image_block_name):

    original = bpy.data.images[image_block_name]['original']
    return original

def get_original_node_name_render_layer_cache_node(scene_name,cache_node_name):

    original = bpy.data.scenes[scene_name].node_tree.nodes[cache_node_name]['original']
    return original

def get_original_node_name_from_cache_node(scene_name,cache_node_name):
    try:
        if 'original' in bpy.data.scenes[scene_name].node_tree.nodes[cache_node_name]:
            return get_original_node_name_render_layer_cache_node(scene_name,cache_node_name)
        else:
            return get_original_node_name_from_image_block_name(bpy.data.scenes[scene_name].node_tree.nodes[cache_node_name].image.name)
    except:
        return None

def get_all_cache_nodes_for_this_node(scene_name,node_name,get_muted = True):

    if get_muted:
        mute_check = '(n.mute == True or n.mute == False)'
    else:
        mute_check = 'n.mute == False'
    if bpy.data.scenes[scene_name].node_tree.nodes[node_name].type == 'R_LAYERS':

        cache_node_names = [n.name for n in bpy.data.scenes[scene_name].node_tree.nodes
                            if n.type =='IMAGE'
                            and n.image
                            and eval(mute_check)
                            and file_tools.get_file_name_from_path(n.image.filepath).startswith('RL_')
                            and get_original_node_name_from_cache_node(scene_name,n.name) == node_name]
    else:
        cache_node_names = [n.name for n in bpy.data.scenes[scene_name].node_tree.nodes
                            if n.type =='IMAGE'
                            and n.image
                            and eval(mute_check)
                            and file_tools.get_file_name_from_path(n.image.filepath).startswith('CH_')
                            and get_original_node_name_from_image_block_name(n.image.name) == node_name]

    return cache_node_names

def refresh_compositor(scene_name,is_cache_render = True, anim = False, execution_mode = 'EXEC_DEFAULT',write_still_frame = False,lock_ui = True,use_the_viewport = False):
    locked = False
    if lock_ui and not bpy.context.scene.render.use_lock_interface:
        locked = True
        bpy.context.scene.render.use_lock_interface = True

    bpy.data.scenes[scene_name]['cache_render'] = True
    print('about to render compositor only')
    bpy.ops.render.render(execution_mode,animation = anim,write_still=write_still_frame,use_viewport=use_the_viewport)
    del bpy.data.scenes[scene_name]['cache_render']

    if locked:
         bpy.context.scene.render.use_lock_interface = False
    print('refreshed compositor')

def mute_everything_but_upstream_and_downstream_nodes(selected_node_names,scene_name,mute_upstream=False,mute_downstream=False):
    """mutes everything apart from upstreams, downstream, composite nodes, and the selected nodes.  setting either mute_upstream or mute_downstream to True will also mute those nodes"""

    nodes_not_to_mute = set([n.name for n in bpy.data.scenes[scene_name].node_tree.nodes if n.type == 'COMPOSITE'])
    for n in selected_node_names:
        nodes_not_to_mute.add(n)
        if not mute_upstream:
            upstream_nodes,unused = node_tools.get_upstream_nodes(n,scene_name)
            nodes_not_to_mute.update(upstream_nodes)
        if not mute_downstream:
            downstream_nodes,unused = node_tools.get_downsttream_nodes(n,scene_name)
            nodes_not_to_mute.update(downstream_nodes)

    muted_nodes = set()
    for n in bpy.data.scenes[scene_name].node_tree.nodes:
        if n.name not in nodes_not_to_mute and n.mute == False:
            n.mute = True
            muted_nodes.add(n.name)

    return muted_nodes
