
from . import comp_operations
from ..dont_register import scene_helpers,checks
import bpy
from hashlib import md5
from os import path
import glob
import os

def get_absolute_path(inpath):
    abspath = get_safe_path(bpy.path.abspath(inpath))

    return abspath

def get_safe_path(inpath):
    """returns the path with the path seperators changed to match the OS"""
    safepath = bpy.path.native_pathsep(inpath)
    return safepath

def get_file_name_from_path(inpath):
    filename = bpy.path.basename(get_safe_path(inpath))
    return filename

def file_exists(file_path):

    if file_path is not None and path.exists(file_path):
        return True
    else:
        return False

def get_exr(scene_name,node_name,frame = None,get_newest_rl_cache_if_not_found = True,check_exists = True):
    """for rl cache nodes it'll get the newest current frame exr or if there isn't one for the current frame, gets the newest for any frame,
    for render layer cache it'll use a wildcard in place of the rl_ or rlt
    for standard cache it'll get the exr for the current frame only"""

    if frame is None:
        frame = bpy.data.scenes[scene_name].frame_current
    if checks.is_a_cache_node(node_name,scene_name):
        original_node_name = comp_operations.get_original_node_name_from_cache_node(scene_name,node_name)
    else:
        original_node_name = node_name

    node_type = None
    if bpy.data.scenes[scene_name].node_tree.nodes[original_node_name].type == 'R_LAYERS':
        node_type = 'R_LAYERS'
        path_of_exr = get_cache_file_path_from_original_node_name(scene_name,original_node_name)

        path_of_exr_with_frame = f'{path_of_exr}{str(frame).zfill(4)}.exr'
        if check_exists:
            list_of_files = glob.glob(path_of_exr_with_frame)
            if not list_of_files and get_newest_rl_cache_if_not_found:

                path_of_exr_with_wildcard = f'{path_of_exr}*'
                list_of_files = glob.glob(path_of_exr_with_wildcard)

    if not check_exists and node_type == 'R_LAYERS':
        return path_of_exr_with_frame
    if list_of_files:
        return max(list_of_files, key= path.getmtime)

def get_cache_file_path_from_original_node_name(scene_name,original_node_name, frame=None):
    """returns the absolute path including file name (excluding the frame number and externsion),
    it works for all node types (render layer, image node, normal original node)
    """

    directory = '//test_cache/'
    directory = bpy.data.scenes[scene_name].node_tree.nodes[original_node_name].scene.the_cache_folder
    directory = get_absolute_path(directory)    
    folder = 'render_layers'    
    fname = md5(f'{bpy.data.scenes[scene_name].node_tree.nodes[original_node_name].scene.name}{bpy.data.scenes[scene_name].node_tree.nodes[original_node_name].layer}'.encode('ascii')).hexdigest()
    fname = f'RL_{fname}'

    full_path = f"{directory}{folder}/{fname}_"
    full_path = get_safe_path(full_path)
    return full_path

def get_files(inpath):
    return glob.glob(inpath)
