from . import file_tools
from ..dont_register import checks
from . import comp_operations

import bpy

def get_downsttream_nodes(start_node_name,scene_name, downstream_nodes= None,end_nodes = None):
    """returns all downstream nodes as a set"""
    if downstream_nodes is None:
        downstream_nodes = set()
    if end_nodes is None:
        end_nodes = set()
    if checks.node_has_no_linked_inputs(start_node_name,scene_name):
        end_nodes.add(start_node_name)
    else:
        for i in bpy.data.scenes[scene_name].node_tree.nodes[start_node_name].inputs:

            for link in i.links:
                downstream_nodes.add(link.from_node.name)
                get_downsttream_nodes(link.from_node.name,scene_name,downstream_nodes,end_nodes)
    return downstream_nodes,end_nodes

def get_downsttream_nodes_including_downstream_of_cache_nodes(start_node_name,scene_name, downstream_nodes= None,end_nodes = None,cache_nodes = None):
    if downstream_nodes is None:
        downstream_nodes = set()
    if end_nodes is None:
        end_nodes = set()
    if cache_nodes is None:
        cache_nodes = set()

    if start_node_name in bpy.data.scenes[scene_name].node_tree.nodes:
        if checks.node_has_no_linked_inputs(start_node_name,scene_name):
            end_nodes.add(start_node_name)
            if checks.is_a_cache_node(start_node_name,scene_name):
                cache_nodes.add(start_node_name)
                original_node_name = comp_operations.get_original_node_name_from_cache_node(scene_name,start_node_name)
                if original_node_name is not None and original_node_name not in downstream_nodes:
                    downstream_nodes.add(original_node_name)
                    get_downsttream_nodes_including_downstream_of_cache_nodes(original_node_name,scene_name,downstream_nodes,end_nodes,cache_nodes)
        else:
            for i in bpy.data.scenes[scene_name].node_tree.nodes[start_node_name].inputs:

                for link in i.links:
                    downstream_nodes.add(link.from_node.name)
                    get_downsttream_nodes_including_downstream_of_cache_nodes(link.from_node.name,scene_name,downstream_nodes,end_nodes,cache_nodes)
    return downstream_nodes,end_nodes,cache_nodes

def get_upstream_nodes(start_node_name,scene_name, upstream_nodes= None,upstream_cache_nodes = None):

    if upstream_nodes is None:
        upstream_nodes = set()
    if upstream_cache_nodes is None:
        upstream_cache_nodes = set()

    if not checks.node_has_linked_outputs(start_node_name,scene_name):
        cache_nodes = comp_operations.get_all_cache_nodes_for_this_node(scene_name,start_node_name)
        for node in cache_nodes:
            if node not in upstream_nodes:
                upstream_cache_nodes.add(node)
                upstream_nodes.add(node)
                get_upstream_nodes(node,scene_name,upstream_nodes,upstream_cache_nodes)
    else:
        for i in bpy.data.scenes[scene_name].node_tree.nodes[start_node_name].outputs:

            for link in i.links:
                if link.to_node.name not in upstream_nodes:
                    upstream_nodes.add(link.to_node.name)
                    get_upstream_nodes(link.to_node.name,scene_name,upstream_nodes,upstream_cache_nodes)
    return upstream_nodes,upstream_cache_nodes

def get_render_result_exr(scene_name,nodes_to_check = None,get_muted = False):
    if nodes_to_check is None or len(nodes_to_check) <1:
        nodes_to_check = [n.name for n in bpy.data.scenes[scene_name].node_tree.nodes]
    if get_muted:
        mute_check = '(bpy.data.scenes[scene_name].node_tree.nodes[n].mute == True or bpy.data.scenes[scene_name].node_tree.nodes[n].mute == False)'
    else:
        mute_check = 'bpy.data.scenes[scene_name].node_tree.nodes[n].mute == False'

    cache_node_names = [
                        n for n in nodes_to_check
                        if 'temporal_parent' not in bpy.data.scenes[scene_name].node_tree.nodes[n]
                        and bpy.data.scenes[scene_name].node_tree.nodes[n].type == 'IMAGE'
                        and bpy.data.scenes[scene_name].node_tree.nodes[n].image
                        and ('original' in bpy.data.scenes[scene_name].node_tree.nodes[n] or 'original' in bpy.data.scenes[scene_name].node_tree.nodes[n].image)
                        and eval(mute_check)
                        and file_tools.get_file_name_from_path(bpy.data.scenes[scene_name].node_tree.nodes[n].image.filepath).startswith('RL_')
                       ]
    return cache_node_names
