from . import file_tools
from . import node_tools
import bpy
from ...variables import addon_variables



def update_rendered_image_paths(scene_name):
    """loads exr for this frame and then refreshes the compositor by performing a cycles render with the render layer node muted to avoid rendering 3d viewport"""

    images_processed = set()
    cache_nodes = set(node_tools.get_render_result_exr(scene_name,get_muted=False))

    if cache_nodes:
        for c in cache_nodes:
            img_name = bpy.data.scenes[scene_name].node_tree.nodes[c].image.name
            if img_name not in images_processed:
                new_exr_path = file_tools.get_exr(scene_name,c,check_exists=False)
                if new_exr_path:
                    bpy.data.images[img_name].filepath = new_exr_path
                    images_processed.add(img_name)

        ''''ignore this commented out section, usually render.render is called after updating image datablocks to force the playhead to wait for the compisitor to fully execute, but this didn't appear to be the problem here'''
        #refresh the compositor by rendering with all render layer nodes disabled (so that only the compositor tree is re-rendered, not the 3d scene)
        #This is to force the playhead to wait until the compositor has finished processing.  Without this, each frame will only display a partially finished image in the backdrop.
        #print('call comp refresh from scene_helpser.py line 25')
        #comp_operations.refresh_compositor(scene_name,False)

    addon_variables.first_playback_frame = None
    return True


def mute_what_can_be_muted_before_playback(scene_name):
    '''mutes the render layers nodes to avoid 3d viewport being rendered, also mutes the composite node and other nodes not downstream of the viewer node'''

    sel = [n for n in bpy.data.scenes[scene_name].node_tree.nodes if n.select == True and n.type == 'VIEWER']
    if not sel:
        sel = [n for n in bpy.data.scenes[scene_name].node_tree.nodes if n.type == 'VIEWER']
    downstream = set()
    end_nodes = set()
    for a in sel:
        downstream,end_nodes =  node_tools.get_downsttream_nodes(a.name,scene_name,downstream,end_nodes)
        downstream.add(a.name)
    nodes_not_to_mute = downstream
    #get list of nodes that arent downstream of the viewer node (these can be muted as they wont impact the result)
    nodes_to_mute = set([n.name for n in bpy.data.scenes[scene_name].node_tree.nodes if n.name not in nodes_not_to_mute and n.mute == False])
    rl_nodes = set([rl.name for rl in bpy.data.scenes[scene_name].node_tree.nodes if rl.type == 'R_LAYERS' and rl.mute == False])
    nodes_to_mute.update(rl_nodes)
    for n in nodes_to_mute:
        bpy.data.scenes[scene_name].node_tree.nodes[n].mute = True
    addon_variables.nodes_muted_at_beginning_of_playback = nodes_to_mute
    return True

def un_mute_playback_muted_cache_nodes(scene_name,clear_variable = False,unmute_render_layer = True,unmute_file_output_nodes = True):
    to_unmute = addon_variables.nodes_muted_at_beginning_of_playback
    if to_unmute is not None:
        for n in to_unmute:
            if n in bpy.data.scenes[scene_name].node_tree.nodes:
                if not unmute_render_layer and bpy.data.scenes[scene_name].node_tree.nodes[n].type == 'R_LAYERS' :
                    pass
                elif not unmute_file_output_nodes and bpy.data.scenes[scene_name].node_tree.nodes[n].type == 'OUTPUT_FILE':
                    pass
                else:
                    bpy.data.scenes[scene_name].node_tree.nodes[n].mute = False

        if clear_variable:
            addon_variables.nodes_muted_at_beginning_of_playback = None
        return True
    return False





